
package ejbay.clients;

import ejbay.account.AccountHomeRemote;
import ejbay.account.AccountRemote;
import ejbay.account.UserObj;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
import java.io.*;
import java.util.Properties;

public class Client_1 
{
    public static void main(String [] args) 
    {
	BufferedReader cmdLine=new BufferedReader(new InputStreamReader(System.in));

        // Check to make sure that the necessary input arguments are provided
        if(args.length != 1 || (args[0].compareTo("") == 0))
        {
            System.out.println("Error: Incorrect parameters passed");
            System.out.println(" usage: java Client_1 <JNDI server>");
            System.exit(1);
        }

        try 
        {
            int cmd = -1;
            Context jndiContext = getInitialContext(args[0]);
            Object ref = jndiContext.lookup("AccountHomeRemote");
            AccountHomeRemote home = (AccountHomeRemote)
                PortableRemoteObject.narrow(ref,AccountHomeRemote.class);   
            AccountRemote account_1 = home.create();
    
            System.out.println("Welcome!");
            System.out.println("Please enter your command or 'help' to see a "
                               + "list of commands");

            while(cmd != CMD_QUIT)
            {
                String cmdString;
                String arguments;
                int startIndex;

                System.out.print("> ");
                // Read input from command line
                cmdString = cmdLine.readLine();
                // Parse input for the request
                cmd = parseRequest(cmdString);                
                startIndex = cmdString.indexOf(' ');
                if(startIndex > 0)
                    arguments = cmdString.substring(startIndex + 1);
                else
                    arguments = new String();
                // Handle that request
                handleRequest(cmd, arguments, account_1);                    
            }

        } 
        catch (java.rmi.RemoteException re){re.printStackTrace();}
        catch (javax.naming.NamingException ne){ne.printStackTrace();}
        catch (javax.ejb.CreateException ce){ce.printStackTrace();}
        catch (java.io.IOException ioe){ioe.printStackTrace();}
        
    }

    /* Get a new initial context that sets up the client to find
     * the JNDI server.
     *
     * @param jndiServer The URL of the JNDI server to connect to
     * @return A handle to the context that JNDI should use
     */
    public static Context getInitialContext(String jndiServer) throws javax.naming.NamingException
    {
        Properties env = new Properties();
        
        env.setProperty(Context.INITIAL_CONTEXT_FACTORY,
                        "org.jnp.interfaces.NamingContextFactory");
        env.setProperty(Context.URL_PKG_PREFIXES, 
                        "org.jboss.naming:org.jnp.interfaces");
        env.setProperty(Context.PROVIDER_URL, jndiServer);
        
        return new InitialContext(env);
    }

    /* Take in a command line request and find what request is being made
     * 
     * @param req The input request
     * @return The integer command value of the request
     */
    public static int parseRequest(String req)
    {
        if(req.equalsIgnoreCase("help"))
            return CMD_HELP;
        if(req.equalsIgnoreCase("quit") || req.equalsIgnoreCase("exit"))
            return CMD_QUIT;
        if(req.regionMatches(true, 0, "acctview", 0, 8))
            return CMD_ACCTVIEW;
        
        return CMD_UNKNOWN;
    }

    /* Performs the necessary action and function call for a given command
     * message.
     *
     * @param cmd Command Message ID number to process
     * @param args The argument list that this command requires (if any)
     * @param acct The account to perform the request on
     */
    public static void handleRequest(int cmd, String args, AccountRemote acct) throws java.rmi.RemoteException
    {
        if(cmd == CMD_QUIT)
            return;
        
        if(cmd == CMD_UNKNOWN)
        {
            System.out.println("Error processing request!  Check your input and try again");
            return;
        }
        if(cmd == CMD_HELP)
        {
            printHelp();
            return;
        }
        if(cmd == CMD_ACCTVIEW)
        {
            viewAcct(acct, args);
            return;
        }

    }

    /* Prints the help message which contains a list of all possible command that this
     * shell will process.
     */
    public static void printHelp()
    {
        System.out.println("");
        System.out.println("List of available commands:");
        System.out.println(" help - display this message");
        System.out.println(" quit - exit this prompt");
        System.out.println(" acctview <account info> - view the corresponding account");
        System.out.println("");
    }

    public static void viewAcct(AccountRemote acct, String args) throws java.rmi.RemoteException
    {
        try
        {
            UserObj user = acct.AcctView(args);
            
            if(null == user)
                System.out.println("No user found matching: " + args);
            else
                System.out.println("Found("+args+"):\n" + user.toString());
        }catch(java.rmi.ServerException rse){
            if((rse.getMessage()).endsWith("java.lang.IllegalArgumentException"))
                System.out.println("Error: No user name entered");
            else
                rse.printStackTrace();
        }

        return;
    }

    // Define constants for the different commands that are processed
    private static int CMD_UNKNOWN = -1;
    private static int CMD_QUIT = 0;
    private static int CMD_HELP = 1;
    private static int CMD_ACCTVIEW = 2;
}
